@@ -16,8 +16,13 @@ static void AlarmHandler();
 void AlarmSet(time)
     int time;
 {
+    struct sigaction handler;
+
     alarmPending = 1;
-    sigset(SIGALRM, AlarmHandler);
+    handler.sa_handler = AlarmHandler;
+    sigemptyset(&handler.sa_mask);
+    handler.sa_flags = 0;
+    sigaction(SIGALRM, &handler, NULL);
     ualarm(1000 * time, 0);
 }
 
