@@ -36,13 +36,7 @@
  * It doesn't keep PERFECT time, but it seems reasonably close
  * for reasonable word speeds on my slow SUN IPC!
  *
- * This code is (ridiculously) heavily modified from morse.c from the Reno UNIX
- * distribution. I (Joe) also used slightly modified versions of a subroutine
- * from Richard Ottolini at Unocal for Sun workstation tone generation.
- * Scott Seligman at Stanford added support for other sorts of devices and
- * made several other changes. John Shalamskas helped test and made comments.
- *
- * For a more detailed history, see the HISTORY file
+ * This program has a long history, see the HISTORY file for details.
  */
 
 /*
@@ -272,6 +266,7 @@ int             firsttime, notdoneyet;
 int             yourchar;
 float           randexp, randnum;
 extern time_t   time ();
+struct sigaction handler;
 
     if (argc == 1 && isatty (fileno (stdin)))
     {
@@ -297,10 +292,17 @@ extern time_t   time ();
 	printf ("      Set the total number of words (groups) to generate.\n");
 	printf ("-C \'STRING\' (default all available characters)\n");
 	printf ("      Select characters to send from this STRING only.\n");
-	printf ("-w words_per_minute (default %g)\n-f frequency_in_hertz (default %g)\n-v volume (zero to one, rather nonlinear, default %g)\n",
-		WORDS_PER_MINUTE, FREQUENCY, VOLUME);
-	printf ("-g alternate_frequency (default %g)\n      (toggles via control-G in input FILE at a word break)\n", FREQUENCY2);
+	printf ("-w words_per_minute (default %g)\n", WORDS_PER_MINUTE);
+	printf ("      actual overall sending speed\n");
 	printf ("-F Farnsworth_character_words_per_minute\n");
+	printf ("      If specified, characters are sent at this speed, with extra\n");
+	printf ("      spaces inserted to bring the overall speed down to the -w\n");
+	printf ("      value. Ignored if not higher than -w.\n");
+	printf ("-f frequency_in_hertz (default %g)\n", FREQUENCY);
+	printf ("-v volume (zero to one, rather nonlinear, default %g)\n",
+		      VOLUME);
+	printf ("-g alternate_frequency (default %g)\n", FREQUENCY2);
+	printf ("      (toggles via control-G in input FILE at a word break)\n");
 	printf ("-e    leave off the <SK> sound at the end\n");
 	printf ("-c    complain about illegal characters instead of just ignoring them\n");
 	printf ("-b    print each word before doing it\n");
@@ -613,10 +615,14 @@ extern time_t   time ();
 
     time(&starttime);
 
-    sigset (SIGINT, die);
-    sigset (SIGTERM, die);
-    sigset (SIGQUIT, die);
-    sigset (SIGTSTP, suspend);
+    handler.sa_handler = die;
+    sigemptyset(&handler.sa_mask);
+    handler.sa_flags = 0;
+    sigaction (SIGINT, &handler, NULL);
+    sigaction (SIGTERM, &handler, NULL);
+    sigaction (SIGQUIT, &handler, NULL);
+    handler.sa_handler = suspend;
+    sigaction (SIGTSTP, &handler, NULL);
 
     if (testing || typeaway)
     {
@@ -624,11 +630,11 @@ extern time_t   time ();
     }
 
 /*
- * Do .25 seconds of silence initially to give the workstation time to
+ * Do 0.25 seconds of silence initially to give the workstation time to
  * get settled after the stress of starting this program and opening
  * up everything.
  */
-    tone (frequency, .25, 0.);
+    tone (frequency, 0.25, 0);
     toneflush ();
 
     if (typeaway)
@@ -1704,9 +1710,9 @@ toneflush (void)
 
 #include <sys/ioctl.h>
 #include <fcntl.h>
-#include <termio.h>
-struct termio   oldtermgtty;
-struct termio   termgtty;
+#include <termios.h>
+struct termios oldtermgtty;
+struct termios termgtty;
 static char    *terminal = "/dev/tty";
 static int      termfd;
 static int      termopen = 0;
@@ -1718,14 +1724,14 @@ openterminal (void)
     /* get parameters and open terminal */
 
     termfd = open (terminal, O_RDWR | O_NDELAY, 0);
-    ioctl (termfd, TCGETA, &termgtty);
+    tcgetattr(termfd, &termgtty);
     oldtermgtty = termgtty;
     if (typeaway != LETMESEE)
 	termgtty.c_lflag &= ~(ECHO | ECHOE | ECHOK | ECHONL);
     termgtty.c_lflag &= ~ICANON;
     termgtty.c_cc[VMIN] = 1;
     termgtty.c_cc[VTIME] = 0;
-    ioctl (termfd, TCSETAW, &termgtty);
+    tcsetattr(termfd, TCSADRAIN, &termgtty);
     termopen = 1;
 }
 
@@ -1752,7 +1758,7 @@ int             n;
 static void
 closeterminal (void)
 {
-    ioctl (termfd, TCSETAW, &oldtermgtty);
+    tcsetattr(termfd, TCSADRAIN, &oldtermgtty);
     close (termfd);
 }
 
@@ -1774,7 +1780,12 @@ cleanup (void)
 static void
 suspend (int sig)
 {
-    sigset (SIGTSTP, suspend);
+    struct sigaction handler;
+
+    sigemptyset(&handler.sa_mask);
+    handler.sa_flags = 0;
+    handler.sa_handler = suspend;
+    sigaction (SIGTSTP, &handler, NULL);
     cleanup ();
     kill (getpid (), SIGSTOP);
     if (termopen)
